# coding: utf-8

from pyspark import SparkContext

from pyspark.mllib.recommendation import Rating
from pyspark.mllib.recommendation import ALS
from pyspark.mllib.recommendation import MatrixFactorizationModel

from pyspark.sql import HiveContext
from pyspark.sql import Row
from pyspark.sql.types import *

import numpy as np
import getopt
import sys
from hdfs import *

# initialize spark context
sc = SparkContext("local", "movie_recommandation_step3")
sqlContext = HiveContext(sc)
sqlContext.sql('use courses')
client=Client("http://master:50070")
 
def trainModel(tablename, modelpath):

  # import and process rating data
  rawData = sqlContext.sql('select * from '+tablename)

  ratings=rawData.map(lambda line: Rating(int(line.userid), int(line.movieid), float(line.rating)))

  cf_model=ALS.train(ratings,50,10,0.01, nonnegative=False, seed=123456)

  if(client.status(modelpath,strict=False)==None):
   cf_model.save(sc, modelpath)

  
def main(argv):
 
 tablename='user_movie'
 modelpath='/user/cda/model/movie_recommendation/cf_model'
 
 try:
  opts,args = getopt.getopt(argv[1:], 't:m:', ['tablename=','modelpath='])
  for param,value in opts:
   if param in ('-t', '--tablename'):
    tablename=value
   elif param in ('-m', '--modelpath'):
    modelpath=value
   else:
    print('no param defined, sys will use default values')
 except:
  print('no entry param entered')
  
 trainModel(tablename,modelpath)
  
  
#param specification: -t/--tablename (table name of etl result of step 1)

if __name__=='__main__':
 main(sys.argv)