# coding: utf-8

from pyspark import SparkContext

from pyspark.mllib.recommendation import Rating
from pyspark.mllib.recommendation import ALS
from pyspark.mllib.recommendation import MatrixFactorizationModel
from pyspark.mllib.evaluation import RegressionMetrics
from pyspark.mllib.evaluation import RankingMetrics

from pyspark.sql import HiveContext
from pyspark.sql import Row
from pyspark.sql.types import *

import getopt
import sys
import numpy as np


# initialize spark context
sc = SparkContext("local", "evaluation_step4")
sqlContext = HiveContext(sc)
sqlContext.sql('use courses')

def evaluate(modelpath, tablename, key):
 
 modelname=modelpath.split('/')[-1]

 # MSE\RMSE\MAE
 rawData = sqlContext.sql('select * from '+tablename)

 ratings=rawData.map(lambda line: Rating(int(line.userid), int(line.movieid), float(line.rating)))
 
 cf_model=MatrixFactorizationModel.load(sc,modelpath)
 
 actual=ratings.map(lambda x: ((x.user, x.product), x.rating))
 prediction = cf_model.predictAll(actual.map(lambda x:x[0])).map(lambda x: ((x.user, x.product), x.rating))
 
 actual_prediciton=actual.join(prediction)
 
 metrics =RegressionMetrics(actual_prediciton.map(lambda x: x[1]))
 
 MSE=metrics.meanSquaredError
 RMSE=metrics.rootMeanSquaredError
 MAE=metrics.meanAbsoluteError
 
 print('mse\rmse\mae evaluation job done')
 # KP\MAP
 #productIDs= cf_model.productFeatures().map(lambda x: x[0]).collect()
 #productMatrix = cf_model.productFeatures().map(lambda x: x[1]).collect()
 
 #estRatings = cf_model.userFeatures().map(lambda x: (x[0], list(np.dot(productMatrix, x[1])))).map(lambda x: (x[0], zip(x[1], productIDs))).map(lambda x: (x[0], sorted(x[1] ,key=(lambda m: m[0]), reverse=True))).map(lambda x: (x[0], [i[1] for i in x[1]]))
 
 #userMovies = ratings.map(lambda r:(r.user, r.product)).groupByKey().mapValues(list)
 
 #predictionAndLabels = estRatings.join(userMovies).map(lambda x: x[1])
 
 #rankingMetrics = RankingMetrics(predictionAndLabels)
 
 #MAP=rankingMetrics.meanAveragePrecision
 #PK=rankingMetrics.precisionAt(key)
 
 rdd=sc.parallelize([[modelname,MSE, RMSE, MAE]])
 
 sqlContext.sql('drop table if exists evaluateresult')
 sqlContext.sql('create table evaluateresult(modelname string, MSE float, RMSE float, MAE float)')
 df=sqlContext.createDataFrame(rdd, ['modelname','MSE','RMSE','MAE'])
 
 df.insertInto('evaluateresult')
 

def main(argv):

 modelpath='/user/cda/model/movie_recommendation/cf_model'
 key=20
 tablename='user_movie'
 
 
 try:
  opts,args = getopt.getopt(argv[1:], 'm:k:t:', ['modelpath=','key=','tablename='])
  for param,value in opts:
   if param in ('-m', '--modelpath'):
    modelpath=value
   elif param in ('-k', '--key'):
    key=value
   elif param in ('-t', '--tablename'):
    tablename=value
   else:
    print('no param defined, sys will use default values')
 except:
  print('no entry param entered')
  
 evaluate(modelpath, tablename,key)
  

if __name__=='__main__':
 main(sys.argv)
 