# coding: utf-8

from pyspark import SparkContext

from pyspark.mllib.recommendation import Rating
from pyspark.mllib.recommendation import ALS
from pyspark.mllib.recommendation import MatrixFactorizationModel

from pyspark.sql import HiveContext
from pyspark.sql import Row
from pyspark.sql.types import *

import numpy as np
import getopt
import sys

# initialize spark context
sc = SparkContext("local", "movie_recommandation_step5")
sqlContext = HiveContext(sc)
sqlContext.sql('use courses')

def getUserrecommend(modelpath,movieid, movietable,recommendtable):
 
 movie_df=sqlContext.sql('select movieid, title from '+movietable)
 movie_rdd=movie_df.map(lambda x: (int(x.movieid), x.title))
 movie=movie_rdd.collectAsMap()
 
 cf_model=MatrixFactorizationModel.load(sc,modelpath)
 topK_users=cf_model.recommendUsers(movieid, 10)
 
 movies_recommend=map(lambda x: (x.user ,x.product, movie[x.product], x.rating), topK_users)
 
 sqlContext.sql('drop table if exists '+recommendtable)
 sqlContext.sql('create table '+recommendtable+'(userid int, movieid int, movie_title string, rating float)')
 df=sqlContext.createDataFrame(movies_recommend, ['userid','movieid','movie_title', 'rating'])
 df.insertInto(recommendtable, False)


def main(argv):

 modelpath='/user/cda/model/movie_recommendation/cf_model'
 movieid=456
 movietable='movie'
 recommendtable='user_recommend'
 
 try:
  opts,args = getopt.getopt(argv[1:], 'm:v:t:r:', ['modelpath=','movieid=','movietable=','recommendtable='])
  for param,value in opts:
   if param in ('-m', '--modelpath'):
    modelpath=value
   elif param in ('-v', '--movieid'):
    userid=value
   elif param in ('-t', '--movietable'):
    movietable=value
   elif param in ('-r','--recommendtable'):
    recommendtable=value
   else:
    print('no param defined, sys will use default values')
 except:
  print('no entry param entered')
  
 getUserrecommend(modelpath, movieid,movietable,recommendtable)


if __name__=='__main__':
 main(sys.argv)