# coding: utf-8

from pyspark import SparkContext

from pyspark.mllib.recommendation import Rating
from pyspark.mllib.recommendation import ALS
from pyspark.mllib.recommendation import MatrixFactorizationModel
from pyspark.mllib.linalg import DenseVector

from pyspark.sql import HiveContext
from pyspark.sql import Row
from pyspark.sql.types import *

import string
import getopt
import sys

# initialize spark context
sc = SparkContext("local", "movie_recommandation_step7")
sqlContext = HiveContext(sc)
sqlContext.sql('use courses')

def cosSim(vectorA, vectorB):
 return vectorA.dot(vectorB) / (vectorA.norm(2)*vectorB.norm(2))

def getsimilarmovie(modelpath,movieid, movietable,resulttable):
 
 movie_df=sqlContext.sql('select movieid, title from '+movietable)
 movie_rdd=movie_df.map(lambda x: (int(x.movieid), x.title))

 movie=movie_rdd.collectAsMap()
 
 cf_model=MatrixFactorizationModel.load(sc,modelpath)
 arr=cf_model.productFeatures().lookup(movieid)[0]
 
 selectedVector=DenseVector(arr)
 
 sims=cf_model.productFeatures().map(lambda x:(x[0], cosSim(selectedVector,DenseVector(x[1])))).top(10,lambda x:x[1])

 similarmovies=map(lambda x:(x[0],movie[x[0]], str(x[1])), sims)

 sqlContext.sql('drop table if exists '+resulttable)
 sqlContext.sql('create table '+resulttable+'(movieid int, movie_title string, similarity string)')
 df=sqlContext.createDataFrame(similarmovies, ['movieid','movie_title','similarity'])
 df.insertInto(resulttable, True)
 

def main(argv):

 modelpath='/user/cda/model/movie_recommendation/cf_model'
 movieid=456
 movietable='movie'
 resulttable='similarmovies'
 
 try:
  opts,args = getopt.getopt(argv[1:], 'm:v:t:r:', ['modelpath=','movieid=','movietable=','resulttable='])
  for param,value in opts:
   if param in ('-m', '--modelpath'):
    modelpath=value
   elif param in ('-v', '--movieid'):
    userid=value
   elif param in ('-t', '--movietable'):
    movietable=value
   elif param in ('-r','--resulttable'):
    resulttable=value
   else:
    print('no param defined, sys will use default values')
 except:
  print('no entry param entered')
  
 getsimilarmovie(modelpath, movieid,movietable,resulttable)
  

if __name__=='__main__':
 main(sys.argv)